require 'uri'
require 'acoustid/api/request/param_attributes'
require 'acoustid/api/request/param_validations'

module Acoustid
  module API
    module Request
      
      class Base
        
        extend ParamAttributes
        extend ParamValidations
        
        class << self
          
          def base_uri
            URI.parse('http://api.acoustid.org/v2/')
          end
          
          def path(value=nil)
            @path = value.to_s.strip unless value.nil?
            
            @path
          end
          
        end
        
        attr_reader :params
        
        # Response format
        param :format, validate: :validate_format, serialize: :serialize_string
        
        def initialize(params={})
          raise TypeError, 'params must respond to :to_hash or :to_h' unless params.respond_to?(:to_hash) || params.respond_to?(:to_h)
          params = params.to_hash rescue params.to_h
          
          params.each { |param, value| send("#{param}=", value) }
        end
        
        def uri
          uri = self.class.base_uri.dup
          uri.path += self.class.path
          
          uri
        end
        
        def valid?
          self.class.required_params.none? do |required_param|
            param = send(required_param)
            
            param.nil? || param.empty?# rescue true
          end
        end
        
        
      end
      
    end
  end
end
