require 'acoustid/api/request/base'

module Acoustid
  module API
    module Request
      
      class Lookup < Base
        
        path 'lookup'
        
        # JSONP callback, only applicable if you select the jsonp format
        param :json_callback, name: 'jsoncallback', serialize: :serialize_string
        
        # Application's API key
        param :client, required: true, serialize: :serialize_string
        
        # Duration of the whole audio file in seconds
        # Only required if trackid is not present
        param :duration, validate: :validate_duration, serialize: :serialize_integer
        
        # Audio fingerprint data
        # Only required if trackid is not present
        param :fingerprint, serialize: :serialize_string
        
        # Track ID (UUID)
        # Only required if duration and fingerprint are not present
        param :track_id, name: 'trackid', serialize: :serialize_string
        
        # Returned metadata
        param :meta, validate: :validate_meta, serialize: :serialize_meta
        
        # Different logic is needed for the validation of this API method
        def valid?
          invalid_duration = duration.nil?
          invalid_fingerprint = fingerprint.nil? || fingerprint.empty?
          invalid_track_id = track_id.nil? || track_id.empty?
          
          invalid_duration_and_fingerprint = invalid_duration && invalid_fingerprint
          
          super && (!invalid_duration_and_fingerprint && invalid_track_id || invalid_duration_and_fingerprint && !invalid_track_id)
        end
        
      end
      
    end
  end
end
