module Acoustid
  module API
    module Request
      
      module ParamValidations
        
        VALID_META_VALUES = %W[recordings recordingids releases releaseids releasegroups releasegroupids tracks compress usermeta sources]
        
        def serialize_string(value)
          value.nil? ? nil : value.to_s.strip
        end
        
        def serialize_integer(value)
          value.nil? ? nil : value.to_i
        end
        
        def serialize_meta(value)
          value.nil? ? nil : value.to_a.collect { |value| value.to_s.strip }
        end
        
        def validate_format(value)
          raise ArgumentError, '' unless %W[json jsonp xml].include?(value.to_s.strip)
        end
        
        def validate_duration(value)
          raise TypeError, 'duration must respond to :to_i' unless value.respond_to?(:to_i)
        end
        
        def validate_meta(value)
          raise TypeError, 'meta must respond to :to_a or be nil' unless value.nil? || value.respond_to?(:to_a)
          raise TypeError, "meta must only include #{ VALID_META_VALUES.join(', ') }" unless value.nil? || serialize_meta(value).all? { |meta| VALID_META_VALUES.include?(meta) }
          
          # TODO
        end
        
      end
      
    end
  end
end
